Index: channels/chan_sip.c
===================================================================
--- channels/chan_sip.c	(revision 109324)
+++ channels/chan_sip.c	(working copy)
@@ -723,7 +723,10 @@
 static enum st_refresher global_st_refresher; /*!< Session-Timer refresher                        */
 static int global_min_se;                     /*!< Lowest threshold for session refresh interval  */
 static int global_max_se;                     /*!< Highest threshold for session refresh interval */
+static int global_force_dtmf_relay = FALSE;   /*!< If the RFC2833 DTMF relay should be forced */
+static int global_force_dtmf_relay_pt = 101;  /*!< Default payload type for emulated telephone-event */
 
+
 /*@}*/ 
 
 /*! \name Object counters @{
@@ -6832,6 +6835,13 @@
 
 	/* Now gather all of the codecs that we are asked for: */
 	ast_rtp_get_current_formats(newaudiortp, &peercapability, &peernoncodeccapability);
+	/* Forcibly add a telephone-event if needed */
+	if (global_force_dtmf_relay && !(peernoncodeccapability & AST_RTP_DTMF)) {
+		ast_rtp_set_m_type(newaudiortp, global_force_dtmf_relay_pt);
+		found_rtpmap_codecs[last_rtpmap_codec++] = global_force_dtmf_relay_pt;
+		ast_rtp_set_rtpmap_type(newaudiortp, global_force_dtmf_relay_pt, "audio", "telephone-event", 0);
+		peernoncodeccapability |= AST_RTP_DTMF;
+	}
 	ast_rtp_get_current_formats(newvideortp, &vpeercapability, &vpeernoncodeccapability);
 	ast_rtp_get_current_formats(newtextrtp, &tpeercapability, &tpeernoncodeccapability);
  
@@ -20273,6 +20283,9 @@
 
 	global_matchexterniplocally = FALSE;
 
+	global_force_dtmf_relay = 0;
+	global_force_dtmf_relay_pt = 101;
+
 	/* Copy the default jb config over global_jbconf */
 	memcpy(&global_jbconf, &default_jbconf, sizeof(struct ast_jb_conf));
 
@@ -20361,6 +20374,18 @@
 			global_relaxdtmf = ast_true(v->value);
 		} else if (!strcasecmp(v->name, "vmexten")) {
 			ast_copy_string(default_vmexten, v->value, sizeof(default_vmexten));
+		} else if (!strcasecmp(v->name, "rtp_force_dtmf_relay")) {
+			if ((global_force_dtmf_relay = ast_true(v->value)))
+				ast_verbose("RTP DTMF relaying will be enforced\n");
+			else
+				ast_verbose("RTP DTMF relaying will not be enforced\n");
+		} else if (!strcasecmp(v->name, "rtp_force_dtmf_relay_pt")) {
+			sscanf(v->value, "%d", &global_force_dtmf_relay_pt);
+			if (global_force_dtmf_relay_pt < 96 || global_force_dtmf_relay_pt > 255) {
+				ast_verbose("RTP forced DTMF relay payload type is not valid: %d. Using default (101)\n", global_force_dtmf_relay_pt);
+				global_force_dtmf_relay_pt = 101;
+			} else
+				ast_log(LOG_WARNING, "RTP forced DTMF relay payload type is %d\n", global_force_dtmf_relay_pt);
 		} else if (!strcasecmp(v->name, "rtptimeout")) {
 			if ((sscanf(v->value, "%d", &global_rtptimeout) != 1) || (global_rtptimeout < 0)) {
 				ast_log(LOG_WARNING, "'%s' is not a valid RTP hold time at line %d.  Using default.\n", v->value, v->lineno);
Index: configs/sip.conf.sample
===================================================================
--- configs/sip.conf.sample	(revision 109324)
+++ configs/sip.conf.sample	(working copy)
@@ -631,7 +631,13 @@
 
 ; jblog = no                  ; Enables jitterbuffer frame logging. Defaults to "no".
 ;-----------------------------------------------------------------------------------
+;rtp_force_dtmf_relay=no      ; Force RFC2833 DTMFs to be sent even if peer 
+			      ; hasn't announced support for it. Default: no
 
+;rtp_force_dtmf_relay_pt=101  ; RTP payload type value for enforced RFC2833
+			      ; DTMFs. Default: 101
+
+
 [authentication]
 ; Global credentials for outbound calls, i.e. when a proxy challenges your
 ; Asterisk server for authentication. These credentials override
